/* eslint max-len: 0, no-nested-ternary:0, jsx-a11y/anchor-is-valid:0, react/jsx-closing-bracket-location: 0, jsx-a11y/no-static-element-interactions:0, jsx-a11y/click-events-have-key-events:0 */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { getZipcodeAddress } from 'redux/modules/address';
import { errorModal } from 'redux/modules/modal';
import { isZipcode, isPhone } from 'utils/verify';

/*
Usage:
Parent:
  <AddressForm onRef={c => { this.addressForm = c; }} />
  props: address={} states=[]
Call child method from parent:
  const address = this.addressForm.getAddress();
  const addressErrors = this.addressForm.getErrors();
JP zipcode: 0660066
*/
@connect(({ locales }) => ({
  common: locales.common,
  country: locales.country,
  // loading: address.zipcodeLoading || address.statesLoading // 放到表格外loading
}), {
  getZipcodeAddress,
  errorModal
})
export default class AddressForm extends Component {
  static propTypes = {
    onRef: PropTypes.func.isRequired,
    common: PropTypes.shape(),
    country: PropTypes.shape(),
    states: PropTypes.arrayOf(PropTypes.any),
    address: PropTypes.shape(),
    preName: PropTypes.string,
    show: PropTypes.bool,
    disable: PropTypes.bool,
    getZipcodeAddress: PropTypes.func.isRequired,
    errorModal: PropTypes.func.isRequired
  }

  static defaultProps = {
    common: {},
    country: {},
    states: undefined,
    address: undefined,
    preName: '',
    show: true,
    disable: false
  }

  state = this.initalAddress(this.props.address);

  componentDidMount() {
    this.props.onRef(this);
  }
  componentWillReceiveProps(nextProps) {
    const { address } = nextProps;
    if (nextProps.disable) this.setState(this.initalAddress(address));
  }

  getAddress = () => {
    const { props: { preName }, state } = this;
    return {
      firstname: state[`${preName}firstname`],
      lastname: state[`${preName}lastname`],
      address1: state[`${preName}address1`],
      address2: state[`${preName}address2`],
      country_id: state[`${preName}country_id`],
      state_id: state[`${preName}state_id`],
      state_name: state[`${preName}state_name`],
      city: state[`${preName}city`],
      zipcode: state[`${preName}zipcode`],
      phone: state[`${preName}phone`]
    };
  };
  getErrors = () => {
    const { props: { country, common, preName }, state } = this;
    const errors = [];
    if (!state[`${preName}firstname`]) errors.push(common.fill_firstname);
    if (!state[`${preName}lastname`]) errors.push(common.fill_lastname);
    if (!state[`${preName}address1`]) errors.push(common.fill_address);
    if (!state[`${preName}state_id`]) errors.push(common.fill_state);
    if (country.city && !state[`${preName}city`]) errors.push(common.fill_city);
    if (!state[`${preName}zipcode`]) errors.push(common.fill_zipcode);
    else if (!isZipcode(state[`${preName}zipcode`], country.code)) errors.push(common.invalid_zipcode);
    if (!state[`${preName}phone`]) errors.push(common.fill_phone);
    else if (!isPhone(state[`${preName}phone`], country.code)) errors.push(common.invalid_phone);
    return errors;
  }

  getZipcodeAddress(zipcode) {
    const { state, props } = this;
    const {
      common, country, preName, states = []
    } = props;
    const errors = [];
    if (!zipcode) errors.push(common.fill_zipcode);
    else if (!isZipcode(zipcode, country.code)) errors.push(common.invalid_zipcode);
    if (errors.length) return this.props.errorModal({ errors });
    this.props.getZipcodeAddress(zipcode)
      .then(body => {
        state[`${preName}address1`] = body.address;
        const matchState = states.find(item => item.name === body.state);
        if (matchState) {
          state[`${preName}state_id`] = matchState.id;
          state[`${preName}state_name`] = matchState.name;
        }
        this.setState(state);
      })
      .catch(err => this.props.errorModal(err));
  }

  resetAddress = address => {
    this.setState(this.initalAddress(address));
  }

  valueChange(event) {
    const { preName } = this.props;
    const {
      name, value, options, selectedIndex
    } = event.target;
    this.setState({
      [name]: value,
      ...name === `${preName}state_id` && {
        [`${preName}state_name`]: options[selectedIndex].text
      }
    });
  }

  initalAddress(address = {}) {
    const { country, preName } = this.props;
    return {
      // email: address.email || '',
      [`${preName}firstname`]: address.firstname || '',
      [`${preName}lastname`]: address.lastname || '',
      [`${preName}address1`]: address.address1 || '',
      [`${preName}address2`]: address.address2 || '',
      [`${preName}country_id`]: country.id,
      [`${preName}state_id`]: address.state_id || '',
      [`${preName}state_name`]: address.state_name || '',
      ...country.city && {
        [`${preName}city`]: address.city || ''
      },
      [`${preName}phone`]: address.phone || '',
      [`${preName}zipcode`]: address.zipcode || ''
    };
  }

  render() {
    const {
      common, country, states, show, disable, preName
    } = this.props;
    const { state } = this;
    const autoComplete = __DEVELOPMENT__ ? 'on' : 'off';
    return (
      <ul className="rasterForm" style={{ ...show ? {} : { display: 'none' } }}>
        <li className="p50">
          <div className="input-normal">
            <span className="placeholder">{common.field_firstname}
              <i className="requiredFlag">*</i>
            </span>
            <input autoComplete={autoComplete} type="text" maxLength="200" name={`${preName}firstname`}
              onChange={e => disable ? {} : this.valueChange(e)} disabled={disable}
              value={state[`${preName}firstname`]}
            />
          </div>
        </li>
        <li className="p50">
          <div className="input-normal">
            <span className="placeholder">{common.field_lastname}
              <i className="requiredFlag">*</i>
            </span>
            <input autoComplete={autoComplete} type="text" maxLength="200" name={`${preName}lastname`}
              onChange={e => disable ? {} : this.valueChange(e)} disabled={disable}
              value={state[`${preName}lastname`]}
            />
          </div>
        </li>
        <li className="p50">
          <div className="input-normal">
            <span className="placeholder">{common.field_country}
              <i className="requiredFlag">*</i>
            </span>
            <input autoComplete={autoComplete} type="text" disabled value={country.name} />
          </div>
        </li>
        <li className="p50">
          <div className="input-normal">
            <span className="placeholder">{common.field_zipcode}
              <i className="requiredFlag">*</i>
              {!disable && <a className="buttonFlag"
                onClick={() => this.getZipcodeAddress(state[`${preName}zipcode`])}
              >
                {common.button_verify}
              </a>}
            </span>
            {disable
              ? <input disabled value={state[`${preName}zipcode`]} />
              : <input type="text" name={`${preName}zipcode`} maxLength="20" placeholder={common.placeholder_zipcode}
                value={state[`${preName}zipcode`]} onChange={e => disable ? {} : this.valueChange(e)}
              />}
          </div>
        </li>
        <li className="p50">
          <div className="select-normal">
            <span className="placeholder">{common.field_state}
              <i className="requiredFlag">*</i>
            </span>
            {disable
              ? <input autoComplete={autoComplete} type="text" disabled value={state[`${preName}state_name`]} />
              : <select className="state" name={`${preName}state_id`} value={state[`${preName}state_id`]}
                onChange={e => this.valueChange(e)}
              >
                <option value="" />
                {states && states[0] && states.map(item => (
                  <option key={item.id} value={item.id}>{item.name}</option>
                ))}
              </select>}
          </div>
        </li>
        {country.city && <li className="p50">
          <div className="input-normal">
            <span className="placeholder">{common.field_city}
              <i className="requiredFlag">*</i>
            </span>
            <input autoComplete={autoComplete} type="text" maxLength="200" name={`${preName}city`}
              onChange={e => disable ? {} : this.valueChange(e)} disabled={disable}
              value={state[`${preName}city`]}
            />
          </div>
        </li>}
        <li className="p100">
          <div className="input-normal">
            <span className="placeholder">{common.field_address1}
              <i className="requiredFlag">*</i>
            </span>
            <input autoComplete={autoComplete} type="text" maxLength="200" name={`${preName}address1`}
              onChange={e => disable ? {} : this.valueChange(e)} disabled={disable}
              value={state[`${preName}address1`]} placeholder={common.placeholder_address1}
            />
          </div>
        </li>
        <li className="p100">
          <div className="input-normal">
            <span className="placeholder">{common.field_address2}</span>
            <input autoComplete={autoComplete} type="text" maxLength="200" name={`${preName}address2`}
              onChange={e => disable ? {} : this.valueChange(e)} disabled={disable}
              value={state[`${preName}address2`]} placeholder={common.placeholder_address2}
            />
          </div>
        </li>
        <li className="p100">
          <div className="input-normal">
            <span className="placeholder">{common.field_phone_number}
              <i className="requiredFlag">*</i>
            </span>
            {disable
              ? <input type="text" value={state[`${preName}phone`]} disabled />
              : <input type="text" name={`${preName}phone`} maxLength="200" placeholder={common.placeholder_phone}
                value={state[`${preName}phone`]} onChange={e => disable ? {} : this.valueChange(e)}
              />}
          </div>
        </li>
      </ul>
    );
  }
}
